/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.audiocd;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.forms.InternetSearchForm;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.freedb.Freedb;
import net.sf.dc.processes.http.FreeDBAudioCDUpdate;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

/**
 * Dialog for retrieving information from an audio CD in the CD-Rom drive.
 * Uses FreeDB to get the information based on the discid
 * @see net.sf.dc.freedb.Freedb
 * @see net.sf.dc.processes.http.FreeDBAudioCDUpdate
 * 
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 1.50
 */
public class ImportAudioCDDialog extends InternetSearchForm {
	
    protected JTextField textSearch;
    protected JButton buttonKeywordSearch;
    protected JButton buttonSearchDisc;
    private JComboBox comboServers;
    
    public ImportAudioCDDialog(ItemForm itemForm) {
        super(itemForm, null, itemForm);
        setTitle(DcResources.getText(Repository.labels.lblImportAudioCD));
        setHelpIndex("dc.Inserting.Odb");
    }
    
    public ImportAudioCDDialog(Class clazz, int rowNumber) {
        super(DataCrow.mainFrame, null, MainFrame._INSERTTAB);
        setTitle(DcResources.getText(Repository.labels.lblImportAudioCD));
        setHelpIndex("dc.Inserting.Odb");
    }
    
    public void setSearchString(String s) {
        textSearch.setText(s);
    }
    
    public void setVisible(boolean b) {
        try {
            String server = DcSettings.getValueAsString(Repository.settings.stImdbServer);
            comboServers.setSelectedItem(server);
        } catch (Exception exp) {}
        super.setVisible(b);
    }
    
    protected DcTable getTable() {
        DcTable table = ComponentFactory.getDCTable(DCModules.getModule(DCModules._AUDIOCD), true, false);
        table.resetTable();
        
        int[] fields = {AudioCD._ID, AudioCD._J_LOCATION, AudioCD._I_DISCID, AudioCD._C_DESCRIPTION, AudioCD._G_PICTUREBACK, 
                        AudioCD._G_PICTUREBACK, AudioCD._L_USER_LONGTEXT, AudioCD._M_USER_SHORTTEXT1, AudioCD._N_USER_SHORTTEXT2,
                        AudioCD._O_USER_NUMERIC1, AudioCD._P_USER_NUMERIC2};
        table.removeFields(fields);
        return table;
    }    
    
    public JPanel getSearchPanel() {
        JPanel panel = new JPanel(false);
        panel.setLayout(Layout.getGBL());
        panel.setBackground(color);
        
        textSearch = ComponentFactory.getTextField();
        textSearch.setMinimumSize(new Dimension(100,19));
        
        buttonKeywordSearch =  ComponentFactory.getButton(DcResources.getText(Repository.labels.lblFind));
        buttonKeywordSearch.addActionListener(new SearchKeywordAction());
        
        buttonSearchDisc =  ComponentFactory.getButton(DcResources.getText(Repository.labels.lblCDSearch));
        buttonSearchDisc.addActionListener(new SearchDiscIDAction());
        
        JLabel labelComment = ComponentFactory.getLabel(DcResources.getText(Repository.labels.lblOnlyXareReturned));
        Font font = new Font(ComponentFactory.getStandardFont().getFontName(), Font.ITALIC, 11);
        labelComment.setFont(font);
        
        JLabel labelServer = ComponentFactory.getLabel(DcResources.getText(Repository.labels.lblInternetServer));
        comboServers = ComponentFactory.getComboBox();
        
        String[] servers; 
        try {
            servers = new Freedb("freedb.freedb.org", this).getAvailableServers();
        } catch (Exception exp) {
            new MessageBox(DcResources.getText(Repository.messages.msgFreedbServersProblem),
                           MessageBox._WARNING);
            servers = new String[] {"freedb.freedb.org"};
        }
        
        for (int i = 0; i < servers.length; i++) {
            comboServers.addItem(servers[i]);
        }
        
        buttonStopSearch =  ComponentFactory.getButton(DcResources.getText(Repository.labels.lblStop));
        buttonStopSearch.addActionListener(new StopSearchAction());          
        
        panel.setLayout(Layout.getGBL());          
        panel.add(labelServer,        Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                                      new Insets(5, 5, 5, 5), 0, 0));                         
        panel.add(comboServers,       Layout.getGBC( 1, 0, 1, 1, 80.0, 80.0
                                     ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                                      new Insets(5, 5, 5, 5), 0, 0));
        panel.add(textSearch,         Layout.getGBC( 0, 1, 2, 1, 90.0, 90.0
                                     ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                                      new Insets(5, 5, 5, 5), 0, 0));                         
        panel.add(buttonKeywordSearch,Layout.getGBC( 2, 1, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                                      new Insets(5, 5, 5, 5), 0, 0));
        panel.add(buttonSearchDisc,   Layout.getGBC( 2, 2, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                                      new Insets(5, 5, 5, 5), 0, 0));
        panel.add(buttonStopSearch,   Layout.getGBC( 3, 1, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                                      new Insets(5, 5, 5, 5), 0, 0));         
        
        try {
            String server = DcSettings.getValueAsString(Repository.settings.stImdbServer);
            comboServers.setSelectedItem(server);
        } catch (Exception exp) {}
        
        return panel;
    }
    
    public void setCancelMode() {
        cancelled = false;
        buttonKeywordSearch.setEnabled(false);
        buttonSearchDisc.setEnabled(false);
        buttonStopSearch.setEnabled(true);
    }    
    
    public void setSearchMode() {
        buttonKeywordSearch.setEnabled(true);
        buttonSearchDisc.setEnabled(true);
        buttonStopSearch.setEnabled(false);        
    }
    
    public void startSearch() {
        cancelled = false;
        String keywords = textSearch.getText();
        if (keywords.length() > 0) { 
            startKeywordSearch();
        } else {
            startDiscIDSearch();
        }
    }
    
    public void saveSettings() {
        super.saveSettings();
        DcSettings.setValue(Repository.settings.stFreeDBSearchFormViewSize, getSize());
        DcSettings.setValue(Repository.settings.stFreedbServer,
                            comboServers.getSelectedItem().toString());          
    }
    
    public void startKeywordSearch() {
        saveSettings();
        setCancelMode();
        String keywords = textSearch.getText();
        if (keywords.length() >= 6) { 
            process = new FreeDBAudioCDUpdate(this, DCModules.getModule(DCModules._AUDIOCD), textSearch.getText());
            process.setPriority(Thread.MAX_PRIORITY);
            process.start();            
        } else {
            setSearchMode();
        	new MessageBox(DcResources.getText(Repository.messages.msgSearchStringLength, "6"),
                           MessageBox._WARNING);
        }
    }
    
    public void startDiscIDSearch() {
        saveSettings();
        setCancelMode();
        
    	process = new FreeDBAudioCDUpdate(this, DCModules.getModule(DCModules._AUDIOCD));
    	process.setPriority(Thread.MAX_PRIORITY);
    	process.start();
    }    
    
    /**
     * Initialize the values for this dialog
     */
    protected void setDefaultValues() {
        setSize(DcSettings.getValueAsDimension(
                Repository.settings.stFreeDBSearchFormViewSize));          
    }
    
    private class SearchDiscIDAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            startDiscIDSearch();
        }
    }    
    
    private class SearchKeywordAction implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			startKeywordSearch();
		}
    }
}
