/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import javax.swing.JComboBox;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.renderers.ComboBoxRenderer;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;

/**
 * @version 1.9
 * @since 1.4
 * @author Robert Jan van der Waals
 */
public class DcEditableComboBox extends JComboBox {

    public DcEditableComboBox() {
        super(new Object[] {"", DcResources.getText(Repository.labels.lblEmptySearchValue)});
        
        setFont(ComponentFactory.getStandardFont());
        setBorder(getBorder());
        setRenderer(new ComboBoxRenderer());        
        
        setEditable(true);
    }
    
    public void storeValue() {
        Object selectedItem = getSelectedItem();
        if (selectedItem.toString().trim().length() > 0) {
            boolean unique = true;
            for (int i = 1; i < getItemCount(); i++) {
                Object o = getItemAt(i);
                if (o.equals(selectedItem)) {
                    unique = false;
                    break;
                }
            }
            if (unique) insertItemAt(selectedItem, 2);
        }
    }
}