/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.ComponentInputMap;
import javax.swing.InputMap;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;

import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.helpsystem.JavaHelp;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.SwingUtil;
import net.sf.ext.util.Utilities;

/**
 * Super class for all frames within Data Crow.
 * It only handles setting the Frame as the root frame for modal messages.
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.5
 */
public class DcFrame extends JFrame {
    
    private String helpIndex = null;
    private JFrame parent;
    
    public DcFrame(JFrame parent) {
        SwingUtil.setRootFrame(this);
        setIconImage(IconLibrary._icoMain.getImage());
        
        this.parent = parent;

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                shiftFocus();
            }
        }); 
        
        rootPane.getInputMap().put(KeyStroke.getKeyStroke("F1"), "helpAction");
        rootPane.getActionMap().put("helpAction", new HelpAction());
        
        JRootPane rootPane = getRootPane();
        ActionMap parentActionMap = rootPane.getActionMap();
        ActionMap am = new ActionMap();
        am.setParent(parentActionMap);
        am.put("helpAction", new HelpAction());
        rootPane.setActionMap(am);
         
         
        InputMap parentInputMap = rootPane.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        ComponentInputMap im = new ComponentInputMap(getRootPane());
        try {
            im.setParent(parentInputMap);
        } catch (Exception ignore) {}
        im.put(KeyStroke.getKeyStroke("F1"), "helpAction");   // Bind the F1 key  to the MyAction action
        rootPane.setInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT, im);           
     
    }
    
    private void shiftFocus() {
        SwingUtil.shiftFocus(parent);
    }
    
    public void setVisible(boolean visible) {
        super.setVisible(visible);
        
        setBackground(DcSettings.getValueAsColor(Repository.settings.stColor));
        
        if (visible) {
        	SwingUtil.setRootFrame(this);
        } else {
            shiftFocus();
        }
    }
    
    public void setBackground(Color color) {
        super.setBackground(color);
        Component[] components = getComponents();
        for (int i = 0; i < components.length; i++) {
        	JComponent component = (JComponent) components[i];
            setBackground(component, color);
        }
    }        
    
    /**
     * Set the help ID for this frame
     * @param helpID
     */
    public void setHelpIndex(String helpID) {
        helpIndex =  helpID;
    }
    
    private void setBackground(JComponent component, Color color) {
    	if (component instanceof JLabel || component instanceof JPanel ||
            component instanceof JCheckBox || component instanceof JTabbedPane ||
            component instanceof JRootPane || component instanceof JLayeredPane) {
            
    		component.setBackground(color);
            
            Component[] components = component.getComponents();
            for (int i = 0; i < components.length; i++) {
                JComponent subComponent = (JComponent) components[i];
                setBackground(subComponent, color);
            }
        }
    }
    
    protected void setCenteredLocation() {
        Dimension dim = Utilities.getCenteredWindowLocation(getSize());
        setLocation(dim.width, dim.height);
    }

    private class HelpAction implements Action {
        public boolean isEnabled() {
            return true;
        }

        public void setEnabled(boolean arg0) {}
        public void actionPerformed(ActionEvent arg0) {
            if (helpIndex != null) {
            	JavaHelp.setHelpIndex(helpIndex);
            }
            new JavaHelp();
        }
        
        public void putValue(String arg0, Object arg1) {}
        public void addPropertyChangeListener(PropertyChangeListener arg0) {}
    
        public Object getValue(String arg0) {
            return null;
        }
        
        public void removePropertyChangeListener(PropertyChangeListener arg0) {}
    }    
}
