package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Iterator;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.table.TableColumn;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.ProgramDefinition;
import net.sf.dc.settings.definitions.ProgramDefinitions;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;

public class DcProgramDefinitionsField extends JComponent implements ISettingsComponent {
	    
    private final DcTable programTable = ComponentFactory.getDCTable(true, false);
	private final DcShortTextField extensionField = ComponentFactory.getShortTextField(10);
	private final DcFileField programField = ComponentFactory.getFileField(false, false);
	
    /**
     * Initializes this field
     */
    public DcProgramDefinitionsField() {
        buildComponent();
    }
    
    
    public ProgramDefinitions getDefinitions() {
        return (ProgramDefinitions) getValue();
    }
    
    /**
     * Returns the selected Font (with the chosen size, thickness)
     * Unless the user has chosen otherwise, Arial font size 11 is returned.
     */
    public Object getValue() {
    	ProgramDefinitions definitions = new ProgramDefinitions();
        for (int i = 0; i < programTable.getRowCount(); i++) {
            String extension = (String) programTable.getValueAt(i, 0, true);
            String program = (String) programTable.getValueAt(i, 1, true);
            ProgramDefinition definition = new ProgramDefinition(extension, program);
    		definitions.addDefinition(definition);
    	}

        return definitions;
    }
    
    /**
     * Applies a value to this field
     */
    public void setValue(Object o) {
    	if (o instanceof ProgramDefinitions) {
    		ProgramDefinitions definitions = (ProgramDefinitions) o;
            for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                ProgramDefinition definition = (ProgramDefinition) iter.next();
                Object[] row = {definition.getExtension(), definition.getProgram()}; 
                programTable.addRow(row);
            }
    	}
    }
    
    private void remove() {
        int row = programTable.getSelectedRow();
        if (row > -1)
            programTable.removeRow(row);
    }
    
    private void edit() {
        int row = programTable.getSelectedRow();
        if (row > -1) {
            String extension = (String) programTable.getValueAt(row, 0, true);
            String program = (String) programTable.getValueAt(row, 1, true);
            extensionField.setText(extension);
            programField.setValue(program);
            
            programTable.cancelEdit();
            remove();
        }
    }
    
    private void addDefinition(String extension, String program) {
        if (extension.trim().length() > 0 && program.trim().length() > 0) {
            ProgramDefinitions definitions = getDefinitions();

            String current = definitions.getProgramForExtension(extension);
            if (current != null) {
                new MessageBox(DcResources.getText(Repository.messages.msgProgramAlreadyDefined), MessageBox._WARNING);
            } else {
                Object[] row = {extension, program}; 
                programTable.addRow(row);

                extensionField.setText("");
                programField.setValue(null);
            }
        } else {
            new MessageBox(DcResources.getText(Repository.messages.msgFileOrExtensionNotFilled), MessageBox._WARNING);
        }
    }    
    
    /**
     * Builds this component
     */
    private void buildComponent() {
        setLayout(Layout.getGBL());
        Color color = DcSettings.getValueAsColor(Repository.settings.stColor);
        
        //**********************************************************
        //Input panel
        //**********************************************************          
        JPanel panelInput = new JPanel();
        panelInput.setLayout(Layout.getGBL());
        panelInput.setBackground(color);

        JLabel labelExtention = ComponentFactory.getLabel(DcResources.getText(Repository.labels.lblFileExtension));
        JLabel labelProgram = ComponentFactory.getLabel(DcResources.getText(Repository.labels.lblProgram));

        panelInput.add(labelExtention,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 1, 0, 0, 5), 0, 0));
        panelInput.add(labelProgram,    Layout.getGBC( 1, 0, 1, 1, 10.0, 10.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 0), 0, 0));        
        panelInput.add(extensionField,  Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 1, 0, 0, 5), 0, 0));
        panelInput.add(programField,    Layout.getGBC( 1, 1, 1, 1, 10.0, 10.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 0), 0, 0));        

        //**********************************************************
        //Action panel
        //**********************************************************   
        JPanel panelActions = new JPanel();
        panelActions.setLayout(Layout.getGBL());
        panelActions.setBackground(color);
        JButton buttonAdd = ComponentFactory.getButton(DcResources.getText(Repository.labels.lblAdd));
        JButton buttonRemove = ComponentFactory.getButton(DcResources.getText(Repository.labels.lblRemove));
        
        buttonAdd.addActionListener(new AddProgramAction());
        buttonRemove.addActionListener(new RemoveProgramAction());
        
        panelActions.add(buttonAdd,  	Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                		,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                		 new Insets( 0, 0, 0, 5), 0, 0));
        panelActions.add(buttonRemove,  Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
		        		,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
		        		 new Insets( 0, 0, 0, 0), 0, 0));
        
        //**********************************************************
        //Defined Programs List
        //**********************************************************           
        JScrollPane scroller = new JScrollPane(programTable);
        programTable.addMouseListener(new DoubleClickListener());
        programTable.setColumnCount(2);

        TableColumn columnExtension = programTable.getColumnModel().getColumn(0);
        columnExtension.setCellEditor(new DefaultCellEditor(ComponentFactory.getTextFieldDisabled()));       
        columnExtension.setHeaderValue(DcResources.getText(Repository.labels.lblFileExtension));
        
        TableColumn columnProgram = programTable.getColumnModel().getColumn(1);
        columnProgram.setCellEditor(new DefaultCellEditor(ComponentFactory.getTextFieldDisabled()));
        columnProgram.setHeaderValue(DcResources.getText(Repository.labels.lblProgram));
        
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scroller.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);        
        
        programTable.applyHeaders();
        
        
        //**********************************************************
        //Main panel
        //**********************************************************
        
        add(panelInput,      Layout.getGBC( 0, 0, 2, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets( 0, 0, 0, 0), 0, 0));        
        add(panelActions,    Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                			 new Insets( 0, 0, 0, 0), 0, 0));        
        add(scroller,    	 Layout.getGBC( 0, 2, 2, 1, 20.0, 20.0
                			,GridBagConstraints.SOUTHWEST, GridBagConstraints.BOTH,
                			 new Insets( 0, 0, 0, 0), 0, 0));
        
    }
    
    private class AddProgramAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			addDefinition(extensionField.getText(), programField.getFileAsString());
		}
    }
    
    private class DoubleClickListener implements MouseListener {
        public void mouseClicked(MouseEvent e) {

        }

        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                edit();
            }            
        }
    }    
    

    private class RemoveProgramAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
		    remove();
		}
    }

}
