/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Component;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JToolBar;
import javax.swing.JToolTip;
import javax.swing.SwingUtilities;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.AboutAction;
import net.sf.dc.core.actions.CreateNewAction;
import net.sf.dc.core.actions.CreateReportAction;
import net.sf.dc.core.actions.DeleteAction;
import net.sf.dc.core.actions.HelpAction;
import net.sf.dc.core.actions.ImportAction;
import net.sf.dc.core.actions.InternetUpdateAction;
import net.sf.dc.core.actions.LogAction;
import net.sf.dc.core.actions.NewItemWizardAction;
import net.sf.dc.core.actions.OpenAction;
import net.sf.dc.core.actions.SaveAllAction;
import net.sf.dc.core.actions.SaveSelectedAction;
import net.sf.dc.core.actions.SearchAction;
import net.sf.dc.core.actions.SearchAgainAction;
import net.sf.dc.core.actions.SettingsAction;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

/**
 * The tool bar. All actions for this toolbar can be found in 
 * the net.sf.dc.core.actions package
 *
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 1.6
 */
public class DcToolBar extends JToolBar {

	private final PopupMenu popupMenu = new PopupMenu();

    /**
     * Builds the toolbar
     */
    public DcToolBar() {
        addMouseListener(new ToolBarMouseListener());

        ToolBarButton buttonNewWizard = new ToolBarButton(
                IconLibrary._icoMenuWizard, DcResources.getText(Repository.labels.lblWizard));
        ToolBarButton buttonNew = new ToolBarButton(
                IconLibrary._icoMenuNew, DcResources.getText(Repository.labels.lblNewItem, ""));
        ToolBarButton buttonOpen = new ToolBarButton(
                IconLibrary._icoMenuOpen, DcResources.getText(Repository.labels.lblOpen));
        ToolBarButton buttonDel = new ToolBarButton(
                IconLibrary._icoMenuRemove, DcResources.getText(Repository.labels.lblRemove));
        ToolBarButton buttonSave = new ToolBarButton(
                IconLibrary._icoMenuSave, DcResources.getText(Repository.labels.lblMenuSave));
        ToolBarButton buttonSaveAll = new ToolBarButton(
                IconLibrary._icoMenuSaveAll, DcResources.getText(Repository.labels.lblSaveAll));
        ToolBarButton buttonFind = new ToolBarButton(
                IconLibrary._icoMenuFind, DcResources.getText(Repository.labels.lblSearchForm));
        ToolBarButton buttonFindAgain = new ToolBarButton(
                IconLibrary._icoMenuFindAgain, DcResources.getText(Repository.labels.lblFindAgain));
        ToolBarButton buttonHelp = new ToolBarButton(
                IconLibrary._icoMenuHelp, DcResources.getText(Repository.labels.lblHelp));
        ToolBarButton buttonAbout = new ToolBarButton(
                IconLibrary._icoMenuAbout, DcResources.getText(Repository.labels.lblAbout)); 
        ToolBarButton buttonSettings = new ToolBarButton(
                IconLibrary._icoMenuOption, DcResources.getText(Repository.labels.lblSettings));
        ToolBarButton buttonLog = new ToolBarButton(
                IconLibrary._icoLog, DcResources.getText(Repository.labels.lblLog));

        ToolBarButton buttonReport = new ToolBarButton(
                IconLibrary._icoMenuExport, DcResources.getText(Repository.labels.lblReport));
        
        ToolBarButton buttonImportCD = new ToolBarButton(
                IconLibrary._icoMenuImport, DcResources.getText(Repository.labels.lblImport));
        ToolBarButton buttonInternetUpdate = new ToolBarButton(
                IconLibrary._icoMenuInternetSearch, DcResources.getText(Repository.labels.lblOnlineSearch));
        
        buttonReport.addActionListener(new CreateReportAction());
        buttonNewWizard.addActionListener(new NewItemWizardAction());
        buttonNew.addActionListener(new CreateNewAction());
        buttonOpen.addActionListener(new OpenAction());
        buttonDel.addActionListener(new DeleteAction());
        buttonSave.addActionListener(new SaveSelectedAction());
        buttonSaveAll.addActionListener(new SaveAllAction());
        buttonFind.addActionListener(new SearchAction());
        buttonFindAgain.addActionListener(new SearchAgainAction());
        buttonSettings.addActionListener(new SettingsAction());
        buttonLog.addActionListener(new LogAction());
        buttonAbout.addActionListener(new AboutAction());
        buttonHelp.addActionListener(new HelpAction());
        buttonImportCD.addActionListener(new ImportAction());
        buttonInternetUpdate.addActionListener(new InternetUpdateAction());
        
        buttonNewWizard.setMnemonic(KeyEvent.VK_W);
        buttonNew.setMnemonic(KeyEvent.VK_N);
        buttonOpen.setMnemonic(KeyEvent.VK_O);
        buttonDel.setMnemonic(KeyEvent.VK_D);
        buttonSave.setMnemonic(KeyEvent.VK_S);
        buttonLog.setMnemonic(KeyEvent.VK_L);
        buttonHelp.setMnemonic(KeyEvent.VK_H);
        buttonInternetUpdate.setMnemonic(KeyEvent.VK_I);
        buttonFindAgain.setMnemonic(KeyEvent.VK_A);
        buttonFind.setMnemonic(KeyEvent.VK_F);
        
        add(buttonNewWizard);
		add(buttonNew);
        add(buttonOpen);
        add(buttonSave);
        add(buttonSaveAll);
        add(buttonDel);
        
        addSeparator();
        
        add(buttonFind);
        add(buttonFindAgain);
        
        addSeparator();
        
        add(buttonSettings);
        
        addSeparator();

        add(buttonReport);
        add(buttonImportCD);
        add(buttonInternetUpdate);
        
        addSeparator();
        
        add(buttonLog);
        add(buttonHelp);
        add(buttonAbout);
        
        boolean showLabels = DcSettings.getValueAsBoolean(
                                Repository.settings.stShowMenuBarLabels);
        
        showLabels(showLabels);
	}
    
    /**
     * Displays a popumenu with options for this toolbar
     */
    private void showPopupMenu(int x, int y) {
        popupMenu.show(this, x, y);
    }

    public void toggleLabels() {
        showLabels(!DcSettings.getValueAsBoolean(Repository.settings.stShowMenuBarLabels));
    }
    
    /**
     * Toggles the labels of the tools / buttons
     * @param show
     */
	public void showLabels(boolean show) {
		Component c;
		int i = 0;
		while ((c = getComponentAtIndex(i++)) != null) {
            if (c instanceof ToolBarButton) {
    			ToolBarButton button = (ToolBarButton) c;
    			if (show) {
    				button.setText(button.getToolTipText());
                } else {
    				button.setText(null);
                }
            }
		}
        DcSettings.setValue(Repository.settings.stShowMenuBarLabels, new Boolean(show));
	}
    
    /**
     * Toolbar button component 
     */
	public class ToolBarButton extends JButton {

		private static final long serialVersionUID = 4L;

		public ToolBarButton(Icon icon, String text) {
			super(icon);
			setMargin(new Insets(0, 0, 0, 0));
			setVerticalTextPosition(BOTTOM);
			setHorizontalTextPosition(CENTER);
            setText(text);
            setToolTipText(text);
            setFont(ComponentFactory.getSystemFont());
            addMouseListener(new ToolBarMouseListener());
		}
        
        public JToolTip createToolTip() {
            return new DcMultiLineToolTip();
        }
	}
    
    /**
     * The popup menu with specific sttings for the toolbar 
     */
    public class PopupMenu extends JPopupMenu {

		private static final long serialVersionUID = 6L;

		public PopupMenu() {
            JMenuItem menuToggleLabels = new JMenuItem(DcResources.getText(Repository.labels.lblToggleMenuLabels));

            menuToggleLabels.addActionListener(new ToggleLabelsAction());
            this.add(menuToggleLabels);
        }
    }
    
    /**
     * Toggles the labels for the buttons / tools 
     */
    private class ToggleLabelsAction implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
            boolean showLabels = DcSettings.getValueAsBoolean(Repository.settings.stShowMenuBarLabels);
			showLabels(!showLabels);
		}
    }
    
    /**
     * Listens for a right click on the toolbar
     */
    private class ToolBarMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (SwingUtilities.isRightMouseButton(e)) {
                showPopupMenu(e.getX(), e.getY());
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }    
}