/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JViewport;
import javax.swing.table.TableColumn;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.renderers.CheckBoxTableCellRenderer;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

/**
 * @author Robert Jan van der Waals
 * @version 2.2
 * @since 1.4
 */
public class ReportFieldSelectorPanel extends JPanel {
    
    private DCModule module;
    private final DcTable table;
    
    public ReportFieldSelectorPanel() {
        super();
        
        table = ComponentFactory.getDCTable(false, false);
        setModule(DcSettings.getValueAsInt(Repository.settings.stModule));
        buildComponent();
    }

    public void setModule(int index) {
        module = DCModules.getModule(index); 
    }
    
    public void setDefinitions(ReportFieldDefinitions definitions) {
        table.clearTable();
        for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
            ReportFieldDefinition definition = (ReportFieldDefinition) iter.next();
            table.addRow(definition.getDisplayValues(module));
        }
        table.applyHeaders();
    }
    
    public ReportFieldDefinitions getDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        for (int i = 0; i < table.getRowCount(); i++) {
            DcField field = (DcField) table.getValueAt(i, 0, true);
            boolean enabled = ((Boolean) table.getValueAt(i, 1, true)).booleanValue();
            String alignment = table.getValueAt(i, 2, true).toString();
            definitions.addDefinition(new ReportFieldDefinition(field.getIndex(), enabled, alignment));
        }
        return definitions;
    }
    
    private void buildComponent() {
        setLayout(Layout.getGBL());
        Color color = DcSettings.getValueAsColor(Repository.settings.stColor);
        
        JPanel panel = new JPanel(false);
        panel.setLayout(Layout.getGBL());
        panel.setBackground(color);        

        table.setColumnCount(3);
        
        TableColumn columnField = table.getColumnModel().getColumn(0);
        JTextField textField = new JTextField();
        textField.setEditable(false);
        textField.setEnabled(false);
        textField.setFont(ComponentFactory.getSystemFont());
        textField.setForeground(ComponentFactory.getDisabledColor());
        columnField.setCellEditor(new DefaultCellEditor(textField));       
        
        TableColumn columnEnabled = table.getColumnModel().getColumn(1);
        columnEnabled.setCellEditor(new DefaultCellEditor(new JCheckBox()));
        columnEnabled.setCellRenderer(new CheckBoxTableCellRenderer());
        
        TableColumn columnAlignment = table.getColumnModel().getColumn(2);
        JComboBox comboAlignment = ComponentFactory.getComboBox();
        comboAlignment.addItem(DcResources.getText(Repository.labels.lblLeft));
        comboAlignment.addItem(DcResources.getText(Repository.labels.lblRight));
        comboAlignment.addItem(DcResources.getText(Repository.labels.lblCenter));
        columnAlignment.setCellEditor(new DefaultCellEditor(comboAlignment));

        columnField.setHeaderValue(DcResources.getText(Repository.labels.lblField));
        columnEnabled.setHeaderValue(DcResources.getText(Repository.labels.lblEnabled));
        columnAlignment.setHeaderValue(DcResources.getText(Repository.labels.lblAlignment));
        
        ReportFieldDefinitions definitions = 
            (ReportFieldDefinitions) ReportSettings.getValueAsDefinitionGroup(Repository.reportsettings.stFieldDefinitions);
        setDefinitions(definitions);
        
        JScrollPane scroller = new JScrollPane(table);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scroller.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);

        JLabel labelNav = ComponentFactory.getLabel(DcResources.getText(Repository.labels.lblMove));
        JButton buttonUp = ComponentFactory.getButton(IconLibrary._icoNavigateUp);
        JButton buttonDown = ComponentFactory.getButton(IconLibrary._icoNavigateDown);

        buttonUp.addActionListener(new RowUpAction());
        buttonDown.addActionListener(new RowDownAction());
        
        add(scroller,  Layout.getGBC( 0, 0, 1, 4, 50.0, 50.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 0, 0, 0, 0), 0, 0));
        add(labelNav,  Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 5, 5), 0, 0));
        add(buttonUp,  Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 5), 0, 0));
        add(buttonDown,Layout.getGBC( 1, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 0, 5), 0, 0));        
    }

    private class RowUpAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            table.moveRowUp();
        }
    }
        
    private class RowDownAction implements ActionListener {
        public void actionPerformed( ActionEvent e ) { 
            table.moveRowDown();
        }
    }     
}
