/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.renderers;

import java.awt.Component;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.ToolTipManager;

import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.modules.DCModules;
import net.sf.ext.util.Base64;

/**
 * Table renderer for pictures. Shows an icon if an image is present or
 * just shows the image if the image is very small
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class PictureTableCellRenderer extends DcTableCellRenderer {

    private final int module;
    
    public PictureTableCellRenderer(int module) {
        this.module = module;
    }
    
    protected boolean allowTooltips() {
        return false;
    }
    
    /**
     * Sets the picture. Large images are not displayed themselves,
     * instead an icon is shown to indicate a picture is available.
     * Small pictures are displayed as is (16x16 icons for example)
     * 
     * @param str base64 data stream of the picture
     */
    public void setText(String str) {
        super.setText(str);
        
        if (str.length() > 1) {
            if (module == DCModules._SOFTWARECAT || module == DCModules._BOOKCAT) {
                byte[] bytes = Base64.decode(str.toCharArray());
                ImageIcon icon = new ImageIcon(bytes);
                if (icon.getIconHeight() > 32 && icon.getIconWidth() > 32) {
                    setIcon(IconLibrary._icoPicture);
                    setForeground(this.getBackground());
                } else {
                    setIcon(icon);
                } 
            } else {
                setIcon(IconLibrary._icoPicture);
                setForeground(this.getBackground());
            }
        } else {
        	setIcon(null);
        }
        
        str = null;
    }
    
    /**
     * Renderer implementation. Shows a tooltip, stating 
     * 'picture available' or 'picture unavailable'. As the image
     * is set as a large base64 string, this would not interest
     * the user very much. Instead the text is hidden by applying 
     * the same colors to the fore- and background..
     */
    public Component getTableCellRendererComponent(
                                            JTable table, 
                                            Object value, 
                                            boolean isSelected, 
                                            boolean hasFocus, 
                                            int row,
                                            int column) {

        JComponent c = (JComponent) super.getTableCellRendererComponent(  
                                                            table,  
                                                            value,
                                                            isSelected,
                                                            hasFocus,
                                                            row,
                                                            column);
        ToolTipManager.sharedInstance().setDismissDelay(0);
        setForeground(this.getBackground());
        return c;
    } 
}
