/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.wizards;

import java.awt.GridBagConstraints;
import java.awt.Insets;

import net.sf.dc.console.Layout;
import net.sf.dc.console.audiocd.ImportAudioCDDialog;
import net.sf.dc.console.book.AmazonBookSearchForm;
import net.sf.dc.console.forms.InternetSearchForm;
import net.sf.dc.console.movie.ImdbMovieSearchForm;
import net.sf.dc.console.software.AmazonSoftwareSearchForm;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;

/**
 * @author Robert Jan van der Waals
 * @version 2.2
 * @since 1.4
 */
public class InternetWizardPanel extends WizardPanel {
    
    InternetSearchForm internetSearchForm = null;
    private boolean autoSearch = true; 
    
    public InternetWizardPanel(DCModule module, DcObject dco) {
        build(module, dco);
    }    
    
    public DcObject apply() throws WizardException {
        DcObject result = internetSearchForm.getSelectedData();
        if (result == null) throw new WizardException("No item selected!");
        
        return result;
    }
    
    public String getHelpText() {
        return DcResources.getText(Repository.messages.msgInternetSearch);
    }
    
    public void setObject(DcObject dco) {
        internetSearchForm.setSearchString(dco.getValue(1).toString());
        if (autoSearch) {
            autoSearch = false;
            internetSearchForm.startSearch();
        } 
    }
    
    private void build(DCModule module, DcObject dco) {
        if (module.isAmazonUpdateAllowed()) {  
            if (module.getIndex() == DCModules._SOFTWARE) {
                internetSearchForm = new AmazonSoftwareSearchForm(dco, DataCrow.mainFrame.getSelectedTab());
            } else {
                internetSearchForm = new AmazonBookSearchForm(dco, DataCrow.mainFrame.getSelectedTab());            
            }
        } else if (module.isImdbUpdateAllowed()) {
            internetSearchForm = new ImdbMovieSearchForm(dco, 0, 0);
        } else if (module.isFreedBUpdateAllowed()) {
            internetSearchForm = new ImportAudioCDDialog(this.getClass(), 0);
        } 

        if (internetSearchForm != null) {
            setLayout(Layout.getGBL());
            add(internetSearchForm.getSettingsPanel(), Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
            add(internetSearchForm.getSearchPanel(),   Layout.getGBC(0, 1, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
            add(internetSearchForm.getResultPanel(),   Layout.getGBC(0, 2, 1, 1, 10.0, 10.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
        }
    }
}