package net.sf.dc.core.actions;

import java.awt.event.ActionEvent;
import java.io.File;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.ProgramDefinitions;
import net.sf.ext.util.Utilities;

public class StartProgramAction extends DCAction {
	
    public void actionPerformed(ActionEvent e) {
    	DcObject dco = getModule().getSearchPanel().getSelectedObject();
    	
        if (dco == null) 
            return;
        
    	String filename = dco.getFilename();
    	if (filename != null && filename.trim().length() > 0) {
			File file = new File(filename);
			
			if (file.exists()) {
				String extension = Utilities.getExtension(file);
				
				if (extension == null || extension.length() == 0) {
					new MessageBox(DcResources.getText(Repository.messages.msgInvalidExtension, filename), MessageBox._WARNING);    	
					
				} else {
    				ProgramDefinitions definitions = (ProgramDefinitions) DcSettings.getValue(Repository.settings.stProgramDefinitions);
    				
    				String program = null;
    				if (definitions != null) 
    					program = definitions.getProgramForExtension(extension);

    				if (program == null || program.trim().length() == 0) {
                        new MessageBox(DcResources.getText(Repository.messages.msgNoProgramDefinedForExtension, extension), MessageBox._WARNING);
    				} else {
    					String statement = program + " " + '"' + filename + '"';
                        executeStatement(statement);
    				}
				}    				
			} else { // !exists
                new MessageBox(DcResources.getText(Repository.messages.msgFileDoesNotExist, filename), MessageBox._WARNING);
			}
    	} else { // filename == null
    		new MessageBox(DcResources.getText(Repository.messages.msgNoFilename), MessageBox._WARNING);
    	}
    }
    
    private void executeStatement(String statement) {
        try {
            Process p = Runtime.getRuntime().exec(statement);
            try { 
                p.waitFor();
            } catch (Exception exp) {}
            
        } catch (Exception exp) {
            new MessageBox(DcResources.getText(Repository.messages.msgErrorWhileExecuting, statement), MessageBox._WARNING);
        }
    }
}
