/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.audiocd.AudioCDQuickViewPane;
import net.sf.dc.console.audiocd.ImportAudioCDDialog;
import net.sf.dc.console.audiocd.InsertAudioCDPanel;
import net.sf.dc.console.audiocd.SearchAudioCDPanel;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

/**
 * The Audio CD module
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 *
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.5
 */
public class AudioCDModule extends DCModule {
    
    private ImportAudioCDDialog importAudioCDDialog = null;
    
    
    /**
     * Initializes this module
     */
    public AudioCDModule() {
        super(DCModules._AUDIOCD,
              DcResources.getText(Repository.systemlabels.sysAudioCD),
              "audiocds",
              "aucd",
              " audiocds LEFT JOIN audiotracks ON (audiocds.ID = audiotracks.Album)",
              false,
              false,
              true);
    }

    public ImageIcon getIcon() {
        return IconLibrary._icoAudioCDModule;
    }
    
    public String getObjectName() {
    	return DcResources.getText(Repository.systemlabels.sysAudioCD);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText(Repository.labels.lblTools));
        JMenuItem menuImport = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuImport16,
										        DcResources.getText(Repository.labels.lblAudioCDImport));
        menuImport.addActionListener(new ImportCDAction());
        menu.add(menuImport);
        
    	return menu;
    }
    
    public DcObject getDCObject() {
        return new AudioCD();
    }

    protected void initializeUI() {
        insertPanel = new InsertAudioCDPanel();
        searchPanel = new SearchAudioCDPanel();
    }
        
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsAudioCD;
    }

    
    public void startImport() {
        if (importAudioCDDialog == null) {
            importAudioCDDialog = new ImportAudioCDDialog(InsertAudioCDPanel.class, 0);
        }
        importAudioCDDialog.setVisible(true);
        importAudioCDDialog.startSearch();
    }
    
    public QuickViewPanel getQuickView() {
        if (quickView == null)
            quickView = new AudioCDQuickViewPane();  
        
        return quickView;
    }
    
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
											  String nameOfCD) {
        return null;
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._ID, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._B_ARTIST, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._A_TITLE, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._C_DESCRIPTION, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._D_YEAR, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._E_GENRE, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._F_PICTUREFRONT, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._G_PICTUREBACK , false, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._H_PICTURECD , false, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._I_DISCID , false, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._J_LOCATION , false, 1));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._K_RATING , true, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._L_USER_LONGTEXT, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._M_USER_SHORTTEXT1, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._N_USER_SHORTTEXT2, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._O_USER_NUMERIC1, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(AudioCD._P_USER_NUMERIC2, false, 0));        
        
        return definitions;
    }
    
    /**
     * Initalizes all the fields
     */
    protected void initializeFields() {
        fields = new DcField[17];
        int counter = 0; 
        fields[AudioCD._ID] = new DcField(
                                        counter++,
                                        "ID",
                                        true,
                                        true,
                                        false,
                                        false,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");
        fields[AudioCD._A_TITLE] = new DcField(
                                        counter++,
                                        "Title",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Title");
        fields[AudioCD._B_ARTIST] = new DcField(
                                        counter++,
                                        "Artist",
                                        true,
                                        false,
                                        true,
                                        false,
                                        8000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Artist");
        fields[AudioCD._C_DESCRIPTION] = new DcField(counter++,
                                        "Description",
                                        true,
                                        false,
                                        true,
                                        false,
                                        8000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Description");        
        fields[AudioCD._D_YEAR] = new DcField(counter++,
                                        "Year",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Year");                                       
        fields[AudioCD._E_GENRE] = new DcField(counter++,
                                        "Genre",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Genre");
        fields[AudioCD._F_PICTUREFRONT] = new DcField(counter++,
                                        "Picture Front",
                                        true,
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureFront");
        fields[AudioCD._G_PICTUREBACK] = new DcField(counter++,
                                        "Picture Back",
                                        true,
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureBack");
        fields[AudioCD._H_PICTURECD] = new DcField(counter++,
                                        "Picture CD",
                                        true,
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureCD");
        fields[AudioCD._I_DISCID] = new DcField(counter++,
                                        "Disc ID",
                                        true,
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "DiscID");                                     
        fields[AudioCD._J_LOCATION] = new DcField(counter++,
                                        "Location",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Location");          
        fields[AudioCD._K_RATING] = new DcField(counter++,
                                        "Rating",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._RATINGCOMBOBOX,
                                        Repository.datatypes._INTEGER,
                                        "Rating");  
        
        fields[AudioCD._L_USER_LONGTEXT] = new DcField(counter++, 
                                        "User Long Text Field",
                                        false,
                                        false,
                                        true,
                                        false,
                                        4000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "UserLongText1");
        fields[AudioCD._M_USER_SHORTTEXT1] = new DcField(counter++, 
                                        "User Short Text Field 1",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "UserShortText1");
        fields[AudioCD._N_USER_SHORTTEXT2] = new DcField(counter++, 
                                        "User Short Text Field 2",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "UserShortText2");        
        fields[AudioCD._O_USER_NUMERIC1] = new DcField(counter++, 
                                        "User Numeric Field 1",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "UserInteger1"); 
        fields[AudioCD._P_USER_NUMERIC2] = new DcField(counter++, 
                                        "User Numeric Field 2",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "UserInteger2");         
    }
    
    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
    
    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }       
}
