/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import javax.swing.ImageIcon;
import javax.swing.JMenu;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.audiocd.InsertAudioTrackPanel;
import net.sf.dc.console.audiocd.SearchAudioTrackPanel;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

/**
 * The Audio Track module
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 2.3
 */
public class AudioTrackModule extends DCModule {
    
    /**
     * Initializes this module
     */
    public AudioTrackModule() {
        super(DCModules._AUDIOTRACK,
              DcResources.getText(Repository.systemlabels.sysAudioTrack),
              "audiotracks",
              "autr",
              "audiotracks",
              false,
              false,
              false);
    }

    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysAudioTrack);
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoExisting;
    }
    
    public JMenu getToolsMenu() {
        return null;
    }
    
    /**
     * Return the object managed by this module 
     * @see AudioTrack
     */
    public DcObject getDCObject() {
        return new AudioTrack();
    }
    
    public void startImport() {
    }
    
    public QuickViewPanel getQuickView() {
        return null;
    }
    
    /**
     * Create the insert and search panel
     */
    protected void initializeUI() {
        insertPanel = new InsertAudioTrackPanel();
     	searchPanel = new SearchAudioTrackPanel();
    }
        
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsAudioTrack;
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
                                              String nameOfCD) {
        return null;
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        return new ReportFieldDefinitions();
    }    
    
    /**
     * Initalizes all the fields
     */
    protected void initializeFields() {
        fields = new DcField[11];
        int counter = 0;
        fields[AudioTrack._ID] = new DcField(counter++,
                                        "ID",
                                        true,
                                        true,
                                        false,
                                        false,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");
        fields[AudioTrack._A_TRACKNUMBER] = new DcField(counter++,
                                        "Track",
                                        true,
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Track");
        fields[AudioTrack._B_TITLE] = new DcField(counter++,
                                        "Title",
                                        true,
                                        false,
                                        true,
                                        false,
                                        8000,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Title");
        fields[AudioTrack._C_LYRICS] = new DcField(counter++,
                                        "Lyric",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Lyric");                                       
        fields[AudioTrack._D_PLAYLENGTH] = new DcField(counter++,
                                        "Playlength",
                                        true,
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._TIMEFIELD,
                                        Repository.datatypes._INTEGER,
                                        "PlayLength");
        fields[AudioTrack._E_ALBUM] = new DcField(counter++,
                                        "Album",
                                        true,
                                        true,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._PARENTREFERENCE,
                                        "Album");    
        fields[AudioTrack._F_USER_LONGTEXT] = new DcField(counter++, 
                                        "User Long Text Field",
                                        false,
                                        false,
                                        true,
                                        false,
                                        4000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "UserLongText1");
        fields[AudioTrack._G_USER_SHORTTEXT1] = new DcField(counter++, 
                                        "User Short Text Field 1",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "UserShortText1");
        fields[AudioTrack._H_USER_SHORTTEXT2] = new DcField(counter++, 
                                        "User Short Text Field 2",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "UserShortText2");        
        fields[AudioTrack._I_USER_NUMERIC1] = new DcField(counter++, 
                                        "User Numeric Field 1",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "UserInteger1"); 
        fields[AudioTrack._J_USER_NUMERIC2] = new DcField(counter++, 
                                        "User Numeric Field 2",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "UserInteger2");            
    }
    
    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
}
