/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.movie.InsertMoviePanel;
import net.sf.dc.console.movie.MovieQuickViewPane;
import net.sf.dc.console.movie.SearchMoviePanel;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportMovieCD;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

/**
 * The Movie module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 2.3
 */
public class MovieModule extends DCModule {
    
    private final ImportCDDialog importCDDialog = 
        new ImportCDDialog(DataCrow.mainFrame, 
                           DcResources.getText(Repository.labels.lblMovieFileImport));
    
    /**
     * Initializes this module
     */
    public MovieModule() {
        super(DCModules._MOVIE,
              DcResources.getText(Repository.systemlabels.sysMovie),
              "movies",
              "mov",
              "movies",
              true,
              false,
              false);
    }

    public ImageIcon getIcon() {
        return IconLibrary._icoMovieModule;
    }
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysMovie);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText(Repository.labels.lblTools));
        JMenuItem menuImport = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuImport16,
                                                DcResources.getText(Repository.labels.lblMovieFileImport));
        menuImport.addActionListener(new ImportCDAction());
        menu.add(menuImport);
        
        return menu;
    }
    
    /**
     * Return the object managed by this module 
     * @see Movie
     */
    public DcObject getDCObject() {
        return new Movie();
    }
    
    public QuickViewPanel getQuickView() {
        if (quickView == null)
            quickView = new MovieQuickViewPane();  

        return quickView;            
    }
    
    /**
     * Create the insert and search panel
     */
    protected void initializeUI() {
        insertPanel = new InsertMoviePanel();
        searchPanel = new SearchMoviePanel();
    }
        
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsMovie;
    }
    
    public void startImport() {
        importCDDialog.setModule(DCModules.getModule(DCModules._MOVIE));
        importCDDialog.setVisible(true);
    }
    
    /**
     * Returns the movie CD import process
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmDirs indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */    
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
							     			  String nameOfCD) {

        return new ImportMovieCD(ui,
                                 sourceDir,
                                 confirmDirs,
                                 recursiveDirs,
                                 nameOfCD);
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(Movie._ID, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._A_TITLE, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._B_TITLE_LOCAL, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._C_DESCRIPTION, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._D_WEBPAGE, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._E_GENRE, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._F_ACTORS, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._G_DIRECTOR, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._H_COUNTRY, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._I_YEAR, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._J_PLAYLENGTH, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._K_FILENAME, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._L_VIDEOCODEC, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._M_AUDIOCODEC, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._N_WIDTH, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._O_HEIGHT, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._P_FPS, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._Q_FRAMES, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._R_AUDIOBITRATE, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._S_AUDIOSAMPLINGRATE, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._T_AUDIOCHANNEL, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._U_VIDEOBITRATE , false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._V_PICTUREFRONT , true, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._W_PICTUREBACK , false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._X_PICTURECD , false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._Y_LANGUAGE, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._Z_AUDIOLANGUAGE , false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._1_CERTIFICATION , false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._2_FILESIZE , false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._3_LOCATION , false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._4_RATING , true, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._5_USER_LONGTEXT, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._6_USER_SHORTTEXT1, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._7_USER_SHORTTEXT2, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._8_USER_NUMERIC1, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(Movie._9_USER_NUMERIC2, false, 0));        
        
        return definitions;
    }      
    
    /**
     * Initalizes all the fields
     */
    protected void initializeFields() {
        fields = new DcField[37];
        int counter = 0; 
        fields[Movie._ID] = new DcField(counter++,
                                        "ID",
                                        true,
                                        true,
                                        false,
                                        false,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");
        fields[Movie._A_TITLE] = new DcField(counter++,
                                        "Title",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Title");
        fields[Movie._B_TITLE_LOCAL] = new DcField(counter++,
                                        "Title (local)",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Title2");
        fields[Movie._C_DESCRIPTION] = new DcField(counter++,
                                        "Description",
                                        true,
                                        false,
                                        true,
                                        false,
                                        8000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Description");
        fields[Movie._D_WEBPAGE] = new DcField(counter++,
                                        "Webpage",
                                        true,
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._URLFIELD,
                                        Repository.datatypes._STRING,
                                        "Webpage");                                       
        fields[Movie._E_GENRE] = new DcField(counter++, 
                                        "Genre",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Genre");                                     
        fields[Movie._F_ACTORS] = new DcField(counter++, 
                                        "Actors",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Actors");   
        fields[Movie._G_DIRECTOR] = new DcField(counter++, 
                                        "Director",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Director");     
        fields[Movie._H_COUNTRY] = new DcField(counter++, 
                                        "Country",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Country");    
        fields[Movie._I_YEAR] = new DcField(counter++, 
                                        "Year",
                                        true,
                                        false,
                                        true,
                                        false,
                                        4,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Year");     
        fields[Movie._J_PLAYLENGTH] = new DcField(counter++, 
                                        "Playlength",
                                        true,
                                        false,
                                        false,
                                        false,
                                        10,
                                        ComponentFactory._TIMEFIELD,
                                        Repository.datatypes._INTEGER,
                                        "PlayLength"); 
        fields[Movie._K_FILENAME] = new DcField(counter++, 
                                        "Filename",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Filename");  
        fields[Movie._L_VIDEOCODEC] = new DcField(counter++, 
                                        "Video Codec",
                                        true,
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "VideoCodec");   
        fields[Movie._M_AUDIOCODEC] = new DcField(counter++, 
                                        "Audio Codec",
                                        true,
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "AudioCodec");  
        fields[Movie._N_WIDTH] = new DcField(counter++, 
                                        "Width",
                                        true,
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Width");                                         
        fields[Movie._O_HEIGHT] = new DcField(counter++, 
                                        "Height",
                                        true,
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Height");                                         
        fields[Movie._P_FPS] = new DcField(counter++, 
                                        "Fps",
                                        true,
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Fps");                                         
        fields[Movie._Q_FRAMES] = new DcField(counter++, 
                                        "Frames",
                                        true,
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Frames");                                         
        fields[Movie._R_AUDIOBITRATE] = new DcField(counter++, 
                                        "Audio Bitrate",
                                        true,
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "AudioBitRate");                                         
        fields[Movie._S_AUDIOSAMPLINGRATE] = new DcField(counter++, 
                                        "Audio Sample Rate",
                                        true,
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "AudioSampleRate");                                         
        fields[Movie._T_AUDIOCHANNEL] = new DcField(counter++, 
                                        "Audio Channels",
                                        true,
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "AudioChannels");                                         
        fields[Movie._U_VIDEOBITRATE] = new DcField(counter++, 
                                        "Video Bitrate",
                                        true,
                                        false,
                                        true,
                                        true,
                                        10,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "VideoBitRate");                                         
        fields[Movie._V_PICTUREFRONT] = new DcField(counter++, 
                                        "Picture Front",
                                        true,
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureFront");                                         
        fields[Movie._W_PICTUREBACK] = new DcField(counter++, 
                                        "Picture Back",
                                        true,
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureBack");                                         
        fields[Movie._X_PICTURECD] = new DcField(counter++, 
                                        "Picture CD",
                                        true,
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureCD");  
        fields[Movie._Y_LANGUAGE] = new DcField(counter++, 
                                        "Language",
                                        true,
                                        false,
                                        true,
                                        false,
                                        30,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Language");        
        fields[Movie._Z_AUDIOLANGUAGE] = new DcField(counter++, 
                                        "Audio Language",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "AudioLanguage");   
        fields[Movie._0_SUBTITLELANGUAGE] = new DcField(counter++, 
                                        "Subtitle Language",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "SubtitleLanguage");           
        fields[Movie._1_CERTIFICATION] = new DcField(counter++, 
                                        "Certification",
                                        true,
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Certification");
        fields[Movie._2_FILESIZE] = new DcField(counter++, 
                                        "File Size",
                                        true,
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Filesize");              
        fields[Movie._3_LOCATION] = new DcField(counter++, 
                                        "Location",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Location");       
        fields[Movie._4_RATING] = new DcField(counter++, 
                                        "Rating",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._RATINGCOMBOBOX,
                                        Repository.datatypes._INTEGER,
                                        "Rating");       
        fields[Movie._5_USER_LONGTEXT] = new DcField(counter++, 
                                        "User Long Text Field",
                                        false,
                                        false,
                                        true,
                                        false,
                                        4000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "UserLongText1");
        fields[Movie._6_USER_SHORTTEXT1] = new DcField(counter++, 
                                        "User Short Text Field 1",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "UserShortText1");
        fields[Movie._7_USER_SHORTTEXT2] = new DcField(counter++, 
                                        "User Short Text Field 2",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "UserShortText2");        
        fields[Movie._8_USER_NUMERIC1] = new DcField(counter++, 
                                        "User Numeric Field 1",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "UserInteger1"); 
        fields[Movie._9_USER_NUMERIC2] = new DcField(counter++, 
                                        "User Numeric Field 2",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "UserInteger2");            
    }
    
    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
    
    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }      
}
