/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.musicfile.InsertMusicFilePanel;
import net.sf.dc.console.musicfile.MusicFileQuickViewPane;
import net.sf.dc.console.musicfile.SearchMusicFilePanel;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.musicfiles.MusicFile;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportMusicFilesCD;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

/**
 * The Music module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 2.3
 */
public class MusicFileModule extends DCModule {
    
    private final ImportCDDialog importCDDialog = 
        new ImportCDDialog(DataCrow.mainFrame, DcResources.getText(Repository.labels.lblMusicFileImport));
    
    /**
     * Initializes this module
     */
    public MusicFileModule() {
        super(DCModules._MUSIC,
              DcResources.getText(Repository.systemlabels.sysMusicFile),
              "musicfiles",
              "mufi",
              "musicfiles",
              false,
              false,
              false);
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoMusicModule;
    }
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysMusicFile);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText(Repository.labels.lblTools));
        
        JMenuItem menuImport = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuImport16,
                                                DcResources.getText(Repository.labels.lblMusicFileImport));
        menuImport.addActionListener(new ImportCDAction());

        menu.add(menuImport);
        
        return menu;
    }    
    
    public QuickViewPanel getQuickView() {
        if (quickView == null)
            quickView = new MusicFileQuickViewPane();  

        return quickView;           
    }
    
    /**
     * Return the object managed by this module 
     * @see MusicFile
     */    
    public DcObject getDCObject() {
        return new MusicFile();
    }
        
    /**
     * Create the insert and search panel
     */    
    protected void initializeUI() {
        insertPanel = new InsertMusicFilePanel();
        searchPanel = new SearchMusicFilePanel();
    }
        
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsMusic;
    }
    
    public void startImport() {
        importCDDialog.setModule(DCModules.getModule(DCModules._MUSIC));
        importCDDialog.setVisible(true);
    }
    
    /**
     * Returns the music CD import process
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     * 
     * @return
     */     
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
											  String nameOfCD) {
        return new ImportMusicFilesCD(ui, sourceDir, confirmDirs, recursiveDirs, nameOfCD);
    }
    
    public ReportFieldDefinitions getReportFieldDefinitions() {
        ReportFieldDefinitions definitions = new ReportFieldDefinitions();
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._ID, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._A_TITLE, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._B_ARTIST, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._C_ALBUM, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._D_YEAR, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._E_GENRE, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._F_FILENAME, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._G_PLAYLENGTH, true, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._H_QUALITY, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._I_ENCODING, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._J_LYRIC, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._K_FILESIZE, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._L_LOCATION, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._M_RATING, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._N_USER_LONGTEXT, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._O_USER_SHORTTEXT1, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._P_USER_SHORTTEXT2, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._Q_USER_NUMERIC1, false, 0));
        definitions.addDefinition(new ReportFieldDefinition(MusicFile._R_USER_NUMERIC2, false, 0));        
        
        return definitions;
    }       
    
    /**
     * Initalizes all the fields
     */    
    protected void initializeFields() {
        fields = new DcField[19];
        int counter = 0;
        fields[MusicFile._ID] =         new DcField(counter++, 
                                        "ID",
                                        true,
                                        true,
                                        false,
                                        false,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");
        fields[MusicFile._A_TITLE] = 	new DcField(counter++, 
                                        "Title",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Title");
        fields[MusicFile._B_ARTIST] =  new DcField(counter++, 
                                        "Artist",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Artist");                                       
        fields[MusicFile._C_ALBUM] = 	new DcField(counter++, 
                                        "Album",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Album");                                     
        fields[MusicFile._D_YEAR] 	      = new DcField(counter++, 
                                        "Year",
                                        true,
                                        false,
                                        true,
                                        false,
                                        4,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Year");   
        fields[MusicFile._E_GENRE] 	      = new DcField(counter++, 
                                        "Genre",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Genre");     
        fields[MusicFile._F_FILENAME]   =   new DcField(counter++, 
                                        "Filename",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Filename");
        fields[MusicFile._G_PLAYLENGTH]   = new DcField(counter++, 
                                        "Playlength",
                                        true,
                                        false,
                                        false,
                                        false,
                                        20,
                                        ComponentFactory._TIMEFIELD,
                                        Repository.datatypes._STRING,
                                        "Length");    
        fields[MusicFile._H_QUALITY] 	  = new DcField(counter++, 
                                        "Quality",
                                        true,
                                        false,
                                        true,
                                        true,
                                        4,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Quality");
        fields[MusicFile._I_ENCODING] 	  = new DcField(counter++, 
                                        "Encoding",
                                        true,
                						false,
                                        true,
                                        true,
										255,
										ComponentFactory._SHORTTEXTFIELD,
									    Repository.datatypes._STRING,
                						"Encoding");
        fields[MusicFile._J_LYRIC] 	  	  = new DcField(counter++, 
                                        "Lyric",
                                        true,
										false,
                                        true,
                                        false,
										8000,
										ComponentFactory._LONGTEXTFIELD,
									    Repository.datatypes._STRING,
										"Lyric");
        fields[MusicFile._K_FILESIZE]      = new DcField(counter++, 
                                        "File Size",
                                        true,
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Filesize");           
        fields[MusicFile._L_LOCATION]      = new DcField(counter++, 
                                        "Location",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Location");   
        fields[MusicFile._M_RATING] = new DcField(counter++, 
                                        "Rating",
                                        true,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._RATINGCOMBOBOX,
                                        Repository.datatypes._INTEGER,
                                        "Rating");       
        fields[MusicFile._N_USER_LONGTEXT] = new DcField(counter++, 
                                        "User Long Text Field",
                                        false,
                                        false,
                                        true,
                                        false,
                                        4000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "UserLongText1");
        fields[MusicFile._O_USER_SHORTTEXT1] = new DcField(counter++, 
                                        "User Short Text Field 1",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "UserShortText1");
        fields[MusicFile._P_USER_SHORTTEXT2] = new DcField(counter++, 
                                        "User Short Text Field 2",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "UserShortText2");        
        fields[MusicFile._Q_USER_NUMERIC1] = new DcField(counter++, 
                                        "User Numeric Field 1",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "UserInteger1"); 
        fields[MusicFile._R_USER_NUMERIC2] = new DcField(counter++, 
                                        "User Numeric Field 2",
                                        false,
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "UserInteger2");         
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
    
    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }    
}
