/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.db;

import java.util.Arrays;
import java.util.Collection;

import net.sf.dc.core.objects.DcObject;

/**
 * Specifies on which table, with which fields, the query is run
 *
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 0.5
 */
public class QueryWhat {

    private final Collection columns;
    private final String tableName;
    private final String tableJoin;
    private final String tableShortName;

    public QueryWhat(DcObject dco, String[] fields) {
    	columns = Arrays.asList(fields);
        this.tableName = dco.getTableName();
        this.tableJoin = dco.getTableJoin();
        this.tableShortName = dco.getTableShortName();
    }
    
    /**
     * Returns the list of column names
     */
    public Collection getColumns() {
        return columns;
    }

    public String getJoin() {
    	return tableJoin;
    }
    
    /**
     * Returns the database tablename
     */
    public String getTableName() {
        return tableName;
    }
    
    /**
     * Returns the database tablename
     */
    public String getTableShortName() {
        return tableShortName;
    }    
}
