/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting;

import java.io.File;
import java.util.Iterator;

import javax.swing.ImageIcon;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;
import net.sf.ext.util.StringUtils;
import net.sf.ext.util.Utilities;

import com.lowagie.text.Cell;
import com.lowagie.text.Table;

/**
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 2.1
 */
public class ListReport extends Report {
    
    public ListReport(  ReportingDialog dialog, 
                        DcObject[] data, 
                        File target,
                        int module,
                        ReportFieldDefinitions definitions) {
        
        ReportSettings.setCurrentReport(ReportSettings._LIST_REPORT, module);
        
        this.dialog = dialog;
        this.data = data;
        this.target = target;
        this.definitions = definitions;
    }
    
    public void create() throws Exception {
        initialize();
        
        if (data == null || data.length == 0) return;
        
        // create the table and the header
        Table datatable = getTable(definitions.getEnabledCount());
        datatable.setCellsFitPage(true);
        DcObject dco = data[0];
        setWidths(datatable, dco, definitions);
        
        for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
            ReportFieldDefinition definition = (ReportFieldDefinition) iter.next();
            int field = definition.getIndex();
            
            if (definition.isEnabled()) {
                String label = dco.getField(field).getLabel();
                datatable.addCell(getLabelCell(label, definition));
            }                
        }        
        
        datatable.endHeaders();
        
        for (int j = 0; j < data.length && keepOnRunning; j++) {
            DcObject o = data[j];
            
            dialog.addMessage(DcResources.getText(Repository.messages.msgAddingToReport, o.toString()));
            for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                ReportFieldDefinition definition = (ReportFieldDefinition) iter.next();
                int field = definition.getIndex();

                if (definition.isEnabled()) {
                    String s = o.getDisplayString(field);
                    int fieldType = o.getFieldType(field);
                    
                    if (fieldType == ComponentFactory._RATINGCOMBOBOX) {
                        Object value = o.getValue(field);
                        datatable.addCell(getRatingCell(value, definition));
    
                    } else if (fieldType != ComponentFactory._PICTUREFIELD) {
                        Cell textCell = getTextCell(s, definition);
                        textCell.setMaxLines(5);
                        datatable.addCell(textCell);
                    } else if (o.getValue(field) != null) {
                        s = (String) o.getValue(field);
                        Cell cell = null;
                        if (!StringUtils.isEmpty(s)) {
                            ImageIcon icon = Utilities.base64ToImage(s);
                            cell = getImageCell(icon, definition, 50, 50, true);
                        }
                        if (cell == null)
                            cell = getTextCell(" ", definition);
                            
                        datatable.addCell(cell);
                    }
                } // enabled
            }
            dialog.updateProgressBar(j + 1);
        }
        
        datatable.complete();
        document.add(datatable);
        document.close();
        dialog.addMessage(DcResources.getText(Repository.labels.lblExportHasFinished));
    }
}
