/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.settings;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcColorSelector;
import net.sf.dc.console.components.DcFontSelector;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.Definitions;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;
import net.sf.ext.jflexsett.settings.Setting;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.jflexsett.settings.SettingsFile;
import net.sf.ext.jflexsett.settings.SettingsGroup;
import net.sf.ext.jflexsett.settings.ui.SettingsView;

/**
 * @author Robert Jan van der Waals
 * @version 2.1
 * @since 1.4
 */
public class ReportSettings {
    
    public static final int _CARD_REPORT = 0;
    public static final int _LIST_REPORT = 1;
    
    public static final String _ReportText =  DcResources.getText(Repository.labels.lblReportTextSetting);
    public static final String _ReportLabel = DcResources.getText(Repository.labels.lblReportLabelSetting);    
    public static final String _ReportTextBorderColor =  DcResources.getText(Repository.labels.lblReportTextBorderColor);
    public static final String _ReportLabelBorderColor =  DcResources.getText(Repository.labels.lblReportLabelBorderColor);
    public static final String _ReportHeader = DcResources.getText(Repository.labels.lblReportHeaderSetting);
    public static final String _ReportHeaderFont = DcResources.getText(Repository.labels.lblReportHeaderFont);
    public static final String _ReportHeaderColor = DcResources.getText(Repository.labels.lblReportHeaderColor);
    public static final String _ReportHeaderBackgroundColor = DcResources.getText(Repository.labels.lblReportHeaderBackgroundColor);
    public static final String _ReportFooter = DcResources.getText(Repository.labels.lblReportFooterSetting);
    public static final String _ReportFooterFont = DcResources.getText(Repository.labels.lblReportFooterFont);
    public static final String _ReportFooterColor = DcResources.getText(Repository.labels.lblReportFooterColor);
    public static final String _ReportFooterBackgroundColor = DcResources.getText(Repository.labels.lblReportFooterBackgroundColor);
    public static final String _ReportTextFontSetting = DcResources.getText(Repository.labels.lblReportTextFontSetting);
    public static final String _ReportTextColorSetting = DcResources.getText(Repository.labels.lblReportTextColorSetting);
    public static final String _ReportTextBackgroundColorSetting = DcResources.getText(Repository.labels.lblReportTextBackgroundColorSetting);
    public static final String _ReportLabelFontSetting = DcResources.getText(Repository.labels.lblReportLabelFontSetting);
    public static final String _ReportLabelColorSetting = DcResources.getText(Repository.labels.lblReportLabelColorSetting);
    public static final String _ReportLabelBackgroundColorSetting = DcResources.getText(Repository.labels.lblReportLabelBackgroundColorSetting);    
    
    private static Map cardSettingsMap = new HashMap();
    private static Map listSettingsMap = new HashMap();
    
    private static int currentModule = -1;
    private static int currentReportType = -1;
    
    public ReportSettings() {
    }

    protected void initializeSettings(DCModule module) {
        Settings listSettings = new Settings();
        createGroups(listSettings);
        createSettings(listSettings, _LIST_REPORT);
        File listSettingsFile = new File(DataCrow.baseDir + "data/" + module.getTableName() + "_list_report.properties");
        listSettings.setSettingsFile(listSettingsFile);
        SettingsFile.load(listSettings);
        listSettingsMap.put(module.getName(), listSettings);               
        
        Settings cardSettings = new Settings();
        createGroups(cardSettings);
        createSettings(cardSettings, _CARD_REPORT);
        File cardSettingsFile = new File(DataCrow.baseDir + "data/" + module.getTableName() + "_card_report.properties");
        cardSettings.setSettingsFile(cardSettingsFile);
        SettingsFile.load(cardSettings);
        cardSettingsMap.put(module.getName(), cardSettings);
    }
    
    public static void setCurrentReport(int index, int module) {
        currentReportType = index;
        currentModule = module;
    }
    
    private static Settings getCurrentSettings() {
        DCModule module = DCModules.getModule(currentModule);
        String moduleName = DCModules.getModule(currentModule).getName();
        Settings settings;
        
        if (!cardSettingsMap.keySet().contains(moduleName)) {
            new ReportSettings().initializeSettings(module);  
        }
        
        if (currentReportType == _CARD_REPORT) {
            settings = (Settings) cardSettingsMap.get(moduleName);
        } else {
            settings = (Settings) listSettingsMap.get(moduleName);
        }
        return settings;
    }
    
    public static void save() {
        for (Iterator iter = listSettingsMap.values().iterator(); iter.hasNext(); ) {
            Settings settings = (Settings) iter.next();
            SettingsFile.save(settings);
        }

        for (Iterator iter = cardSettingsMap.values().iterator(); iter.hasNext(); ) {
            Settings settings = (Settings) iter.next();
            SettingsFile.save(settings);
        }
    }
    
    public static SettingsView getView() {
        Settings settings = getCurrentSettings();
        SettingsView view = new SettingsView(null, "Report Settings Panel", settings);
        view.setDisclaimer(DataCrow.baseDir + "icons_system/logo.jpg");
        view.setSize(new Dimension(875, 450));
        Image image = IconLibrary._icoSettings.getImage();
        view.setIconImage(image);
        image.flush(); 
        return view;
    }
    
    public static Definitions getValueAsDefinitionGroup(String key) {
        return getCurrentSettings().getValueAsDefinitionGroup(key);
    }    
    
    public static String[] getValueAsStringArray(String key) {
        return getCurrentSettings().getValueAsStringArray(key);
    }
    
    public static int[] getValueAsIntegerArray(String key) {
        return getCurrentSettings().getValueAsIntegerArray(key);
    }

    public static Font getValueAsFont(String key) {
        return getCurrentSettings().getValueAsFont(key);
    }    
    
    public static int getValueAsInt(String key) {
        return getCurrentSettings().getValueAsInt(key);
    }

    public static boolean getValueAsBoolean(String key) {
        return getCurrentSettings().getValueAsBoolean(key);
    }


    public static Color getValueAsColor(String key) {
        return getCurrentSettings().getValueAsColor(key);
    }
    
    public static String getValueAsString(String key) {
        return getCurrentSettings().getValueAsString(key);
    }

    public static void setValue(String key, Object value) {
        getCurrentSettings().setValue(key, value);
    }

    public static void setStringAsValue(String key, String value) {
        getCurrentSettings().setStringAsValue(key, value);
    }     

    /**
    * Creates the settings group (with their hierarchy)
    */
    protected void createGroups(Settings settings) {
        SettingsGroup reportTextGroup = new SettingsGroup(_ReportText, "dc.Reporting.TextSetting");
        SettingsGroup reportLabelGroup = new SettingsGroup(_ReportLabel, "dc.Reporting.LabelSetting");
        SettingsGroup reportHeaderGroup = new SettingsGroup(_ReportHeader, "dc.Reporting.HeaderSetting");
        SettingsGroup reportFooterGroup = new SettingsGroup(_ReportFooter, "dc.Reporting.FooterSetting");
        
        reportTextGroup.addChild(new SettingsGroup(_ReportTextFontSetting, "dc.Reporting.TextSetting"));
        reportTextGroup.addChild(new SettingsGroup(_ReportTextColorSetting, "dc.Reporting.TextSetting"));
        reportTextGroup.addChild(new SettingsGroup(_ReportTextBackgroundColorSetting, "dc.Reporting.TextSetting"));
        reportTextGroup.addChild(new SettingsGroup(_ReportTextBorderColor, "dc.Reporting.TextSetting"));
        
        reportLabelGroup.addChild(new SettingsGroup(_ReportLabelFontSetting, "dc.Reporting.LabelSetting"));
        reportLabelGroup.addChild(new SettingsGroup(_ReportLabelColorSetting, "dc.Reporting.LabelSetting"));
        reportLabelGroup.addChild(new SettingsGroup(_ReportLabelBackgroundColorSetting, "dc.Reporting.LabelSetting"));
        reportLabelGroup.addChild(new SettingsGroup(_ReportLabelBorderColor, "dc.Reporting.LabelSetting"));

        reportFooterGroup.addChild(new SettingsGroup(_ReportFooterFont, "dc.Reporting.FooterSetting"));
        reportFooterGroup.addChild(new SettingsGroup(_ReportFooterColor, "dc.Reporting.FooterSetting"));
        reportFooterGroup.addChild(new SettingsGroup(_ReportFooterBackgroundColor, "dc.Reporting.FooterSetting"));

        reportHeaderGroup.addChild(new SettingsGroup(_ReportHeaderFont, "dc.Reporting.HeaderSetting"));
        reportHeaderGroup.addChild(new SettingsGroup(_ReportHeaderColor, "dc.Reporting.HeaderSetting"));
        reportHeaderGroup.addChild(new SettingsGroup(_ReportHeaderBackgroundColor, "dc.Reporting.HeaderSetting"));
        
        settings.addGroup(_ReportText, reportTextGroup);
        settings.addGroup(_ReportLabel, reportLabelGroup);
        settings.addGroup(_ReportHeader, reportHeaderGroup);
        settings.addGroup(_ReportFooter, reportFooterGroup);
    }
    
    private void createSettings(Settings settings, int reportType) {
        if (reportType == _CARD_REPORT) {
            settings.addSetting(_ReportText,
                    new Setting(Repository.datatypes._BOOLEAN,
                                Repository.reportsettings.stCreateIndex,
                                Boolean.TRUE,
                                ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblCreateIndex)),
                                DcResources.getText(Repository.tooltips.tpCreateIndex),
                                "",
                                false,
                                true));   
            settings.addSetting(_ReportText,
                    new Setting(Repository.datatypes._INTEGER,
                                Repository.reportsettings.stLabelPosition,
                                new Integer(1),
                                ComponentFactory.getReportTextPositionCombo(),
                                DcResources.getText(Repository.tooltips.tpTextPosition),
                                DcResources.getText(Repository.labels.lblTextPosition),
                                true,
                                true));     
        }

        settings.addSetting(_ReportTextFontSetting,                           // Group key
                new Setting(Repository.datatypes._FONT,                       // Data type
                            Repository.reportsettings.stTextFont,             // Key
                            new Font("Arial", Font.PLAIN, 11),                   // Default value
                            new DcFontSelector(),                               // UI Component
                            DcResources.getText(Repository.tooltips.tpReportTextFont),   // Help text
                            "",                                               // Label text
                            false,                                             // show label ?
                            true));                                           // Add to panel ?    
        settings.addSetting(_ReportTextBackgroundColorSetting,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stTextBackgroundcolor,
                            new Color(247,247,221),
                            new DcColorSelector(false, Repository.reportsettings.stTextBackgroundcolor),
                            DcResources.getText(Repository.tooltips.tpReportTextBackground),
                            "",
                            true,
                            true));
        settings.addSetting(_ReportTextColorSetting,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stTextColor,
                            Color.BLACK,
                            new DcColorSelector(false, Repository.reportsettings.stTextColor),
                            DcResources.getText(Repository.tooltips.tpReportTextColor),
                            "",
                            true,
                            true));
        settings.addSetting(_ReportTextBorderColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stTextBorderColor,
                            Color.BLACK,
                            new DcColorSelector(false, Repository.reportsettings.stTextBorderColor),
                            DcResources.getText(Repository.tooltips.tpReportTextBorderColor),
                            "",
                            true,
                            true));        
        settings.addSetting(_ReportLabelFontSetting,
                new Setting(Repository.datatypes._FONT,
                            Repository.reportsettings.stLabelFont,
                            new Font("Arial", Font.BOLD, 11),
                            new DcFontSelector(),
                            DcResources.getText(Repository.tooltips.tpReportLabelFont),
                            "",
                            false,
                            true));    
        settings.addSetting(_ReportLabelBackgroundColorSetting,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stLabelBackgroundColor,
                            new Color(240,240,191),
                            new DcColorSelector(false, Repository.reportsettings.stLabelBackgroundColor),
                            DcResources.getText(Repository.tooltips.tpReportLabelBackground),
                            "",
                            true,
                            true));
        settings.addSetting(_ReportLabelColorSetting,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stLabelColor,
                            Color.BLACK,
                            new DcColorSelector(false, Repository.reportsettings.stLabelColor),
                            DcResources.getText(Repository.tooltips.tpReportLabelColor),
                            "",
                            true,
                            true));
        settings.addSetting(_ReportLabelBorderColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stLabelBorderColor,
                            Color.BLACK,
                            new DcColorSelector(false, Repository.reportsettings.stLabelBorderColor),
                            DcResources.getText(Repository.tooltips.tpReportLabelBorderColor),
                            "",
                            true,
                            true));         
        settings.addSetting(_ReportHeader,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stShowHeader,
                            Boolean.TRUE,
                            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblShowHeader)),
                            DcResources.getText(Repository.tooltips.tpShowHeader),
                            "",
                            false,
                            true));
        settings.addSetting(_ReportHeader,
                new Setting(Repository.datatypes._STRING,
                            Repository.reportsettings.stHeaderValue,
                            "Report Generated by Data Crow",
                            ComponentFactory.getShortTextField(200),
                            DcResources.getText(Repository.tooltips.tpHeaderValue),
                            DcResources.getText(Repository.labels.lblHeaderValue),
                            true,
                            true));              
        settings.addSetting(_ReportFooter,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stShowFooter,
                            Boolean.TRUE,
                            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblShowFooter)),
                            DcResources.getText(Repository.tooltips.tpShowFooter),
                            "",
                            false,
                            true));    
        settings.addSetting(_ReportFooter,
                new Setting(Repository.datatypes._STRING,
                            Repository.reportsettings.stFooterValue,
                            "page",
                            ComponentFactory.getShortTextField(200),
                            DcResources.getText(Repository.tooltips.tpFooterValue),
                            DcResources.getText(Repository.labels.lblFooterValue),
                            true,
                            true));
        settings.addSetting(_ReportFooterFont,
                new Setting(Repository.datatypes._FONT,
                            Repository.reportsettings.stFooterFont,
                            new Font("dialog.plain", Font.BOLD, 11),
                            new DcFontSelector(),
                            DcResources.getText(Repository.tooltips.tpReportFooterFont),
                            "",
                            false,
                            true));         
        settings.addSetting(_ReportFooterColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stFooterColor,
                            Color.BLACK,
                            new DcColorSelector(false, Repository.reportsettings.stFooterColor),
                            DcResources.getText(Repository.tooltips.tpReportFooterColor),
                            "",
                            true,
                            true));         
        settings.addSetting(_ReportFooterBackgroundColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stFooterBackgroundColor,
                            Color.WHITE,
                            new DcColorSelector(false, Repository.reportsettings.stFooterBackgroundColor),
                            DcResources.getText(Repository.tooltips.tpReportFooterBackgroundColor),
                            "",
                            true,
                            true));     
        settings.addSetting(_ReportHeaderFont,
                new Setting(Repository.datatypes._FONT,
                            Repository.reportsettings.stHeaderFont,
                            new Font("dialog.plain", Font.BOLD, 11),
                            new DcFontSelector(),
                            DcResources.getText(Repository.tooltips.tpReportHeaderFont),
                            "",
                            false,
                            true));         
        settings.addSetting(_ReportHeaderColor, 
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stHeaderColor,
                            Color.BLACK,
                            new DcColorSelector(false, Repository.reportsettings.stHeaderColor),
                            DcResources.getText(Repository.tooltips.tpReportHeaderColor),
                            "",
                            true,
                            true));         
        settings.addSetting(_ReportHeaderBackgroundColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.reportsettings.stHeaderBackgroundColor,
                            Color.WHITE,
                            new DcColorSelector(false, Repository.reportsettings.stHeaderBackgroundColor),
                            DcResources.getText(Repository.tooltips.tpReportHeaderBackgroundColor),
                            "",
                            true,
                            true));      
        
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stTextBorderBottom,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblTextBorderBottom)),
                            "",
                            "",
                            false,
                            true));            
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stTextBorderTop,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblTextBorderTop)),
                            "",
                            "",
                            false,
                            true));            
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stTextBorderLeft,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblTextBorderLeft)),
                            "",
                            "",
                            false,
                            true));            
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stTextBorderRight,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblTextBorderRight)),
                            "",
                            "",
                            false,
                            true));   
        settings.addSetting(_ReportLabel,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stLabelBorderBottom,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblLabelBorderBottom)),
                            "",
                            "",
                            false,
                            true));            
        settings.addSetting(_ReportLabel,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stLabelBorderTop,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblLabelBorderTop)),
                            "",
                            "",
                            false,
                            true));            
        settings.addSetting(_ReportLabel,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stLabelBorderLeft,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblLabelBorderLeft)),
                            "",
                            "",
                            false,
                            true));            
        settings.addSetting(_ReportLabel,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.reportsettings.stLabelBorderRight,
                            Boolean.FALSE,
                            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblLabelBorderRight)),
                            "",
                            "",
                            false,
                            true));                    
        
        
        // INVISIBLE SETTINGS
        if (currentModule != -1) {
            DCModule module = DCModules.getModule(currentModule);
            ReportFieldDefinitions definitions = module.getReportFieldDefinitions();
            
            settings.addSetting(_ReportText,
                    new Setting(Repository.datatypes._DEFINITIONGROUP,
                                Repository.reportsettings.stFieldDefinitions,
                                definitions,
                                null,
                                "",
                                "",
                                false,
                                false));             
        }
        
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._STRING,
                            Repository.reportsettings.stTarget,
                            "",
                            null,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_ReportText,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.reportsettings.stReportType,
                            new Integer(0),
                            null,
                            "",
                            "",
                            false,
                            false));           
    }    
}